﻿namespace U2UConsult.WinRT.ObservableVector.Sample
{
    using Mvvm;
    using Windows.UI.Xaml.Data;
    using Windows.UI.Xaml.Input;

    public class MainViewModel : INotifyPropertyChanged
    {
        private DelegateCommand addCommand;
        private DelegateCommand removeCommand;

        private string selectedDragon;
        private string selectedFavorite;

        public MainViewModel()
        {
            // Collections
            this.Dragons = new ObservableVector<object>() { 
                "Bahamut", 
                "Charizard",
                "Elliot",
                "Faffy",
                "Fafnir",
                "Falkor",
                "Foorwijf",
                "Glaurung",
                "Grendel",
                "Jabberwocky",
                "Leviathan",
                "Mushu",
                "Nessie",
                "Norbert",
                "Orm Embar",
                "Puff",
                "Quetzalcoatl",
                "Ryujin",
                "Scylla",
                "Smaug",
                "Toothless",
                "Valoo",
                "Vermithrax Pejorative",
                "Yoshi"
            };
            this.Favorites = new ObservableVector<object>();

            // Commands
            this.addCommand = new DelegateCommand(this.Add_Executed, this.Add_CanExecute);
            this.removeCommand = new DelegateCommand(this.Remove_Executed, this.Remove_CanExecute);
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public ObservableVector<object> Dragons { get; set; }

        public ObservableVector<object> Favorites { get; set; }

        public string SelectedDragon
        {
            get { return selectedDragon; }
            set
            {
                selectedDragon = value;
                this.PropertyChanged.Raise(this, o => o.SelectedDragon);
                this.addCommand.RaiseCanExecuteChanged();
            }
        }

        public string SelectedFavorite
        {
            get { return selectedFavorite; }
            set
            {
                selectedFavorite = value;
                this.PropertyChanged.Raise(this, o => o.SelectedFavorite);
                this.removeCommand.RaiseCanExecuteChanged();
            }
        }

        public ICommand AddCommand
        {
            get { return this.addCommand; }
        }

        public ICommand RemoveCommand
        {
            get { return this.removeCommand; }
        }

        private bool Add_CanExecute()
        {
            return !string.IsNullOrEmpty(this.selectedDragon);
        }

        private void Add_Executed()
        {
            string item = this.SelectedDragon;
            this.Dragons.Remove(item);
            this.Favorites.Add(item);
        }

        private bool Remove_CanExecute()
        {
            return !string.IsNullOrEmpty(this.selectedFavorite);
        }

        private void Remove_Executed()
        {
            string item = this.SelectedFavorite;
            this.Favorites.Remove(item);
            this.Dragons.Add(item);
        }
    }
}
